C This program converts a matrix of species abundances in such a way that
C the Euclidean distance among rows of the transformed matrix is equal to
C one of the following distances among rows of the original matrix:
C
C    1. chord distance
C    2. chi-square metric
C    3. chi-square distance
C    4. distance between species profiles
C    5. Hellinger distance
C
C Input file: rectangular table of species abundances without 
C             row (= site) or column (= species) identifiers.
C
C References:
C
C Legendre, P. and E. Gallagher. Ecologically meaningful transformations 
C for ordination biplots of species data. Ecology (submitted).
C
C Legendre, P. and L. Legendre, 1998. Numerical Ecology. 2nd English edition.
C Elsevier Science BV, Amsterdam. -- p. 279, pp. 284-286.
C
C                                         Pierre Legendre, May-June 1999
C ----------------------------------------------------------------------
C2345678901234567890123456789012345678901234567890123456789012345678901234567890
      Integer p,pmax,formula
      Parameter (nmax=1000,pmax=1000)
      Real*8 vec(pmax),sx,sx2,mat(nmax,pmax),ypj(pmax),ypp
      Character*79 nom
C
C     call F_Creator('R*ch')
C
      write(*,*) 'Name of input data file?'
      read(*,*) nom
      open(7,file=nom,status='OLD')
      write(*,*) 'How many rows (objects) and columns (species)?'
      read(*,*) n,p
      if(p.gt.pmax) write(*,*)'Too many species. Recompile the program.'
C
    2 write(*,*) 'Choice of data transformation:'
      write(*,*) '  1. chord distance'
      write(*,*) '  2. chi-square metric'
      write(*,*) '  3. chi-square distance'
      write(*,*) '  4. distance between species profiles'
      write(*,*) '  5. Hellinger distance'
      read(*,*) formula
      if((formula.lt.1).or.(formula.gt.5)) goto 2
C
      write(*,*) 'Name of output file?'
      read(*,*) nom
      open (8,file=nom,status='NEW')
C
    4 write(*,*)
      write(*,*) 'Output: '
      write(*,*) '(1) a single line per object, or'
      write(*,*) '(2) 8 species values per line?'
      read(*,*) ichoix
      if((ichoix.lt.1).or.(ichoix.gt.2)) goto 4
C
      goto (10,20,20,40,40) formula
C Chord distance
   10 do 18 i=1,n
      read(7,*) (vec(j), j=1,p)
      sx2=0.0
      do 12 j=1,p
   12 sx2=sx2+vec(j)*vec(j)
      if(sx2.eq.0.0) then
         write(*,100) i
         goto 16
         endif
      sx2=dsqrt(sx2)
      do 14 j=1,p
   14 vec(j)=vec(j)/sx2
   16 if(ichoix.eq.1) then
         write(8,101) (vec(j), j=1,p)
         else
         write(8,102) (vec(j), j=1,p)
         endif
   18 continue
      goto 99
C Chi-square metric; chi-square distance
   20 do 21 j=1,p
   21 ypj(j)=0.0
      do 22 i=1,n
      read(7,*) (mat(i,j), j=1,p)
      do 22 j=1,p
      ypj(j)=ypj(j)+mat(i,j)
   22 continue
      ypp=0.0
      do 23 j=1,p
      if(ypj(j).eq.0.0) write(*,103) j
   23 ypp=ypp+ypj(j)
   
      do 28 i=1,n
      sx=0.0
      do 24 j=1,p
   24 sx=sx+mat(i,j)
      if(sx.eq.0.0) then
         write(*,100) i
         goto 26
         endif
      do 25 j=1,p
      if(ypj(j).eq.0.0) then
         mat(i,j)=0.0
         else
         mat(i,j)=mat(i,j)/(sx*dsqrt(ypj(j)))
         if(formula.eq.3) mat(i,j)=mat(i,j)*dsqrt(ypp)
         endif
   25 continue
   26 if(ichoix.eq.1) then
         write(8,101) (mat(i,j), j=1,p)
         else
         write(8,102) (mat(i,j), j=1,p)
         endif
   28 continue
      goto 99
C Distance between species profiles; Hellinger distance
   40 do 48 i=1,n
      read(7,*) (vec(j), j=1,p)
      sx=0.0
      do 42 j=1,p
   42 sx=sx+vec(j)
      if(sx.eq.0.0) then
         write(*,100) i
         goto 46
         endif
      do 44 j=1,p
      vec(j)=vec(j)/sx
   44 if(formula.eq.5) vec(j)=dsqrt(vec(j))
   46 if(ichoix.eq.1) then
         write(8,101) (vec(j), j=1,p)
         else
         write(8,102) (vec(j), j=1,p)
         endif
   48 continue
C
   99 close (7)
      close (8)
      stop
  100 format(' Line',i4,' has a sum of 0')
  101 format(1000f10.5)
  102 format(8f10.5)
  103 format(' Column',i4,' has a sum of 0')
      end
